#include "huc.h"


#define	SHIPS_IMAGE	0x4000
#define	BULLET_IMAGE	0x4080
#define	POD_IMAGE	0x40C0

#define	XMIN		0
#define	XMAX		224
#define	YMIN		0
#define	YMAXTOP		100
#define	YMAXBOT		210
#define SHIP_Y		210

/* sprite numbers */
#define BULLET1_SPRNUM	9
#define	BULLET2_SPRNUM	10
#define BULLET_SPEED	3

/* types of enemies */
#define	SPRTYPE_POD	0
#define	SPRTYPE_EYES	1
#define	SPRTYPE_FIGHTER	2
#define	SPRTYPE_DRONE	3
#define	SPRTYPE_LEGS	4
#define	SPRTYPE_SAUCER	5
#define	SPRTYPE_CYCLOP	6

/* explode fragments */
#define EXPLOD_FRAGS		50
#define EXPLOD_FRAGSPRITE	11

/********************/
/* Global Variables */
/********************/

/* for each of 8 enemy sprites, keep position information */
int  spr_movptr[8];	/* position in movement list */
int    spr_posx[8];	/* x-position */
int    spr_posy[8];	/* y-position */

int    spr_type[8];	/* POD/EYES, etc. */
int   spr_phase[8];	/* animation sequence */

/* explosion fragments */
int frag_on[EXPLOD_FRAGS];
int frag_x[EXPLOD_FRAGS];
int frag_y[EXPLOD_FRAGS];
int frag_dx[EXPLOD_FRAGS];
int frag_dy[EXPLOD_FRAGS];


/* ship and bullets */
int ship_x;

int bullet1_on;
int bullet2_on;
int bullet1_x;
int bullet1_y;
int bullet2_x;
int bullet2_y;

int score;
int hiscore;




/**************************************/
/* DATA FOR VARIOUS MOVEMENTS AS EGGS */
/**************************************/

/* list of scores to add when shot */
const int  hit_value[7] = { 10, 50, 60, 70, 80, 100, 150 };


/* pointer to all patterns */
const int  movpats[5]  = {	movpat1, movpat2, movpat3, movpat4, movpat5 };

/* diamond shape, clockwise */
const char movpat1[31] = {	0x22,0x22,0x22,0x22,0x22,0x22,0x22,0xA2,
				0xA2,0xA2,0xA2,0xA2,0xA2,0xA2,0xAA,0xAA,
				0xAA,0xAA,0xAA,0xAA,0xAA,0x2A,0x2A,0x2A,
				0x2A,0x2A,0x2A,0x2A,
				0x00,LOW(movpat1),HIGH(movpat1) };

/* diamond shape, counter-clockwise (not yet) */
const char movpat2[31] = {	0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0x2A,
				0x2A,0x2A,0x2A,0x2A,0x2A,0x2A,0x22,0x22,
				0x22,0x22,0x22,0x22,0x22,0xA2,0xA2,0xA2,
				0xA2,0xA2,0xA2,0xA2,
				0x00,LOW(movpat2),HIGH(movpat2) };

/* left and right */
const char movpat3[131] = {	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
				0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
				0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
				0x0A,0x0A,0x0A,0x0A,0x02,0x02,0x02,0x02,
				0x0A,0x0A,0x0A,0x0A,0x02,0x02,0x02,0x02,
				0x0A,0x0A,0x0A,0x0A,0x02,0x02,0x02,0x02,
				0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,
				0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,
				0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,
				0x0A,0x0A,0x0A,0x0A,0x02,0x02,0x02,0x02,
				0x0A,0x0A,0x0A,0x0A,0x02,0x02,0x02,0x02,
				0x0A,0x0A,0x0A,0x0A,0x02,0x02,0x02,0x02,
				0x00,LOW(movpat3),HIGH(movpat3) };

/* clockwise triangle */
const char movpat4[75] = {	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
				0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
				0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
				0x29,0x29,0x29,0x29,0x29,0x29,0x29,0x29,
				0x29,0x29,0x29,0x29,0x29,0x29,0x29,0x29,
				0x29,0x29,0x29,0x29,0x29,0x29,0x29,0x29,
				0xA9,0xA9,0xA9,0xA9,0xA9,0xA9,0xA9,0xA9,
				0xA9,0xA9,0xA9,0xA9,0xA9,0xA9,0xA9,0xA9,
				0xA9,0xA9,0xA9,0xA9,0xA9,0xA9,0xA9,0xA9,
				0x00,LOW(movpat4),HIGH(movpat4) };

/* counter-clockwise triangle */
const char movpat5[75] = {	0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,
				0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,
				0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,
				0xA1,0xA1,0xA1,0xA1,0xA1,0xA1,0xA1,0xA1,
				0xA1,0xA1,0xA1,0xA1,0xA1,0xA1,0xA1,0xA1,
				0xA1,0xA1,0xA1,0xA1,0xA1,0xA1,0xA1,0xA1,
				0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,
				0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,
				0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,
				0x00,LOW(movpat5),HIGH(movpat5) };

/*********************/
/* Program functions */
/*********************/

explode()
{
int i;
int keep_going;

    /* create 32 random fragments cetered in ship's */
    /* center and having random (x,y) directions */

    for (i = 0; i < EXPLOD_FRAGS; i++)
    {
        frag_on[i] = 1;
        frag_x[i] = ship_x + 12 + (rand() & 2);
	frag_y[i] = SHIP_Y - 3;
	frag_dx[i] = (rand() & 15) - 7;
	frag_dy[i] = (rand() & 15);
    }

    keep_going = 1;


    /* as long as some fragments on screen, */
    /* keep displaying explosion */

    while (keep_going)
    {
        keep_going = 0;

	for (i = 0; i < EXPLOD_FRAGS; i++)
	{
            if (frag_on[i] == 0)
	    {
                spr_hide(i + EXPLOD_FRAGSPRITE);
	    }
	    else
	    {
		/* move the fragment by (dx,dy) */
		/* amount, and disable if off-screen */

                frag_x[i] += frag_dx[i];

		if ((frag_x[i] < XMIN) || (frag_x[i] > XMAX))
		{
                    frag_on[i] = 0;
                    continue;
		}

	        frag_y[i] -= frag_dy[i];

		if ((frag_y[i] < YMIN) || (frag_y[i] > YMAXBOT))
		{
                    frag_on[i] = 0;
                    continue;
		}

		/* create feeling of gravity by */
		/* altering the dy amount each frame */

		frag_dy[i]--;

                keep_going = 1;
		

		/* display it */

		spr_set(i + EXPLOD_FRAGSPRITE);
		spr_x(frag_x[i]);
		spr_y(frag_y[i]);
		spr_pattern(BULLET_IMAGE);
		spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
		spr_pal(2);
		spr_pri(1);
	    }
	}

	/* wait for next video frame */
	vsync(0);
        satb_update();
    }
}

spr_move(sprite, x, y)
int sprite;
int x;
int y;
{
    spr_set(sprite);
    spr_x(x);
    spr_y(y);

    spr_pattern(POD_IMAGE);
    spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
    spr_pal(2);
    spr_pri(1);
}

move_bullet()
{
    if (bullet1_on == 1)
    {
        bullet1_y -= BULLET_SPEED;
	if (bullet1_y < YMIN)
	{
            spr_hide(BULLET1_SPRNUM);
	    bullet1_on = 0;
        }
	else
	{
	    spr_set(BULLET1_SPRNUM);
	    spr_x(bullet1_x);
	    spr_y(bullet1_y);
	    spr_pattern(BULLET_IMAGE);
            spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
	    spr_pal(2);
	    spr_pri(1);
	}
    }

    if (bullet2_on == 1)
    {
        bullet2_y -= BULLET_SPEED;
	if (bullet2_y < YMIN)
	{
	    spr_hide(BULLET2_SPRNUM);
	    bullet2_on = 0;
	}
	else
	{
	    spr_set(BULLET2_SPRNUM);
	    spr_x(bullet2_x);
	    spr_y(bullet2_y);
	    spr_pattern(BULLET_IMAGE);
            spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
	    spr_pal(2);
	    spr_pri(1);
	}
    }
}

move_ship()
{
    if (joy(0) & JOY_LEFT)
    {
        if (ship_x > XMIN)
	{
	    ship_x -= 2;
	}
    }

    if (joy(0) & JOY_RGHT)
    {
        if (ship_x < XMAX)
	{
	    ship_x += 2;
	}
    }

    if (joy(0) & JOY_B)
    {
        explode();
    }

    if (joy(0) & JOY_A)
    {
        if ( (bullet1_on != 1) && (bullet2_on) != 1 )
	{
            bullet1_on = 1;
	    bullet1_x = ship_x + 2;
	    bullet1_y = SHIP_Y;

	    bullet2_on = 1;
	    bullet2_x = ship_x + 23;
	    bullet2_y = SHIP_Y;
        }
    }

    spr_set(0);
    spr_x(ship_x);
    spr_y(SHIP_Y);

    spr_pattern(SHIPS_IMAGE);
    spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_32x16|NO_FLIP);
    spr_pal(2);
    spr_pri(1);

    move_bullet();
}

move1egg(ind)
int ind;
{
int movex, movey;
char * chpos;
int * intpos;
char dx, dy;

    chpos = spr_movptr[ind];
    movex = spr_posx[ind];
    movey = spr_posy[ind];

    dx = *chpos & 0x0f;
    dy = *chpos >> 4;

    if (dx & 8)
    {
        dx &= 0x07;
        movex -= dx;

        if (movex < XMIN)
        {
            movex = XMIN;
        }
    }
    else
    {
        movex += dx;

        if (movex > XMAX)
        {
	    movex = XMAX;
        }
    }

    if (dy & 8)
    {
        dy &= 0x07;
        movey -= dy;

        if (movey < YMIN)
        {
            movey = YMIN;
        }
    }
    else
    {
        movey += dy;

        if (movey > YMAXTOP)
        {
	    movey = YMAXTOP;
        }
    }


    chpos++;
    if (*chpos == 0)
    {
        chpos++;
        intpos = chpos;
        chpos = *intpos;
    }

    spr_movptr[ind] = chpos;
    spr_posx[ind]   = movex;
    spr_posy[ind]   = movey;

    spr_move((ind+1), movex, movey);
}


main() {

int movex, movey;
char * chpos;
int * intpos;
char dx, dy;

    hiscore = 0;

    load_sprites(SHIPS_IMAGE,ships,1);

    spr_movptr[0] = &movpat1[0];
    spr_posx[0] = 50;
    spr_posy[0] = 100;

    spr_movptr[1] = &movpat2[0];
    spr_posx[1] = 100;
    spr_posy[1] = 100;

    spr_movptr[2] = &movpat3[0];
    spr_posx[2] = 150;
    spr_posy[2] = 100;

    spr_movptr[3] = &movpat4[0];
    spr_posx[3] = 200;
    spr_posy[3] = 100;

    spr_movptr[4] = &movpat5[0];
    spr_posx[4] = 250;
    spr_posy[4] = 100;

    bullet1_on = 0;
    bullet2_on = 0;
    ship_x = 100;

    score = 0;

    put_string("score:",  0,0);
    put_string("hiscore:", 19,0);
    
    set_sprpal(2,ships_colors);

    set_color(0,0);
    set_color(1,511);

    init_satb();

    while (1) {
        vsync(0);
        satb_update();

	move1egg(0);
	move1egg(1);
	move1egg(2);
	move1egg(3);
	move1egg(4);

	move_ship();

	put_number(score, 5, 6,0);

	put_number(hiscore, 5, 27,0);
    }

}



#incspr(ships, "spores.pcx",0,0,2,3)
#incpal(ships_colors, "spores.pcx",0,1)
